#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import sys
import os
import re

sys.path.append('/Users/robertoneves/Projetos/bravaorm/')

import bravaorm
from model import *

import time
import json

def main(arg):
    start = time.time()

    conn = bravaorm.Connection(db_user="root", db_password="1234", db_host="127.0.0.1", db_port=3306, db_database="gofans", db_charset="utf8mb4", log_level='debug')

    resultado = conn.execute("SELECT * FROM usuarios", "Usuario")

    for usr in resultado:
        print(usr.toJSON())

    print("\n\nFim conversa")
    conn.close()

if __name__ == "__main__":
    main(sys.argv)



# -*- coding: utf-8 -*-

import decimal
import datetime
import json

class ValidateValue:

    def __init__(self, context=None, keyname=None, pk=False, auto_increment=False, fk=False, not_null=False, required=False, max=None, name=None, type=None, format=None, precision=None, scale=None, key=None, reference=None, table=None, intermediate=None, rel_key=None, ref_key=None):
        self.pk = pk
        self.fk = fk
        self.required = required
        self.max = max
        self.name = name
        if name:
            self.type = getattr(__import__(f'model.{name.lower()}', fromlist=[name]), name)
        else:
            self.type = None
        self.auto_increment = auto_increment
        self.not_null = not_null
        self.precision = precision
        self.scale = scale
        self.format = format
        self.key = key
        self.reference = reference
        self.table = table
        self.intermediate = intermediate
        self.rel_key = rel_key
        self.ref_key = ref_key
        self.context = context
        self.keyname = keyname
        self._value = None if not self.__class__ or not 'List' in self.__class__.__name__ else ListType(self.type, context, keyname)

    def __call__(self, **kw):
        return self.type(**kw)

    def __reset__(self):
        self._value = None if not self.__class__ or not 'List' in self.__class__.__name__ else ListType(self.type, self.context, self.keyname)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value


class String(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, str):
                raise Exception("requires string")
            if self.max and len(data) > self.max:
                raise Exception(f"Value too large. The default limit is {self.max}")
        super().__setattr__(attr, data)


class Int(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if self.max and len(str(data)) > self.max:
                raise Exception(f"Value too large. The default limit is {self.max}")
            if not isinstance(data, int):
                try:
                    data = int(data)
                except ValueError as e:
                    raise Exception("requires int")
        super().__setattr__(attr, data)


class DateTime(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, datetime.datetime):
                try:
                    data = datetime.datetime.strptime(data, self.format)
                except ValueError as e:
                    if "unconverted data remain" not in str(e):
                        raise Exception("is not datetime or string is not correct format")
                    pass
        super().__setattr__(attr, data)


class Decimal(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, Decimal):
                try:
                    data = decimal.Decimal(str(round(float(data), self.scale)))
                except ValueError as e:
                    raise Exception("requires Decimal")
        super().__setattr__(attr, data)


class Float(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, float):
                try:
                    data = round(float(data), self.scale)
                except ValueError as e:
                    raise Exception("requires Float")
        super().__setattr__(attr, data)


class Boolean(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, bool):
                try:
                    data = bool(data)
                except ValueError as e:
                    raise Exception("requires Boolean")
        super().__setattr__(attr, data)


class Dict(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if isinstance(data, str):
                try:
                    data = json.loads(data)
                except Exception as e:
                    raise Exception("requires Json String or Dict")
            elif not isinstance(data, dict):                
                raise Exception("requires Dict")
        super().__setattr__(attr, data)

class Json(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if not isinstance(data, dict) and not isinstance(data, str):
                raise Exception("requires Dict or String")
            if isinstance(data, dict):
                data = str(data)
        super().__setattr__(attr, data)


class Obj(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            if data.__class__.__name__ != self.type.__name__:
                raise Exception(f"requires {self.type.__name__} object")
        super().__setattr__(attr, data)


class ObjList(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            self.value.add(data)
        super().__setattr__(attr, data)


class ObjListOfMany(ValidateValue):

    def __init__(self, **attrs):
        super().__init__(**attrs)

    def __setattr__(self, attr, data, check=True):
        if check and attr == "value" and data is not None:
            self.value.add(data)
        super().__setattr__(attr, data)

class ListType(list):

    def __init__(self, type, _context=None, key=None):
        self._type = type
        self._context = _context
        self._key = key
        super(ListType, self).__init__()

    def append(self, item):
        try:

            if not isinstance(item, set) and not isinstance(item, map) and not isinstance(item, list) and item.__class__.__name__ != self._type.__name__:
                raise Exception(f'Item type not is {self._type.__name__}')

            if self._context and not self._key in self._context.__metadata__['relasionships']:
                self._context.__metadata__['relasionships'][self._key] = []

            if isinstance(item, list):
                super(ListType, self).extend(item)
                if (self._context):
                    self._context.__metadata__['relasionships'][self._key].extend(self.toJSON())
            elif isinstance(item, set) or isinstance(item, map):
                super(ListType, self).extend(list(item))
                if (self._context):
                    self._context.__metadata__['relasionships'][self._key].extend(self.toJSON())
            else:
                super(ListType, self).append(item)
                if (self._context):
                    self._context.__metadata__['relasionships'][self._key].append(item.toJSON())

        except Exception as e:
            raise e
        return self

    def add(self, item):
        self.append(item)
        return self

    def toJSON(self):
        try:
            return [item.toJSON() if hasattr(item, "toJSON") else item for item in self]
        except Exception as e:
            raise e



#!/usr/bin/env python

# LIBS
from datetime import datetime
import traceback as trace

# LOGGING
import logging

# CREATE TOP LOG LEVEL
log = logging.getLogger("bravaorm")

# ADITIONAL LOG MANAGER
from .colored_log import ColoredFormatter
from .colors import *

def Logging(log_level='error'):
    
     if not len(log.handlers):

        _level = logging.ERROR
        if log_level == 'debug':
            _level = logging.DEBUG
        elif log_level == 'info':
            _level = logging.INFO
        elif log_level == 'warning':
            _level = logging.WARNING
        elif log_level == 'error':
            _level = logging.ERROR
        elif log_level == 'critical':
            _level = logging.CRITICAL

        _format = "%(levelname)s:\t  %(message)s"

        # Add console handler using our custom ColoredFormatter
        ch = logging.StreamHandler()
        ch.setLevel(_level)
        cf = ColoredFormatter(_format)
        ch.setFormatter(cf)
        log.addHandler(ch)

        # Set log level
        log.setLevel(_level)

def Info(message):
    log.info(message)

def Debug(message):
    log.debug(message)

def Warning(message):
    log.warning(message)

def Error(sector, e, traceback=True):
    _trace = None
    message = ""
    if traceback:
        _trace = trace.format_exc()
        message = f"\n {sector if sector else ''}: {_trace if _trace else ''} \n"
    log.error(f"{message} Message: {e if e else ''}")

def Critical(sector, e, traceback=True):
    _trace = None
    message = ""
    if traceback:
        _trace = trace.format_exc()
        message = f"\n {sector if sector else ''}: {_trace if _trace else ''} \n"
    log.critical(f"{message} Message: {e if e else ''}")



# -*- coding: utf-8 -*-

import sys
import os
import re

import mysql.connector as mariadb
from bravaorm.utils.inflector import Inflector

def reference_name(table, list, name, fk_name):
    __table_referenced = name
    if not __table_referenced in list and __table_referenced != table:
        list.append(__table_referenced)
        return __table_referenced
    else:
        __table_referenced = fk_name
        if __table_referenced.lower().startswith("fk_"):
            __table_referenced = "_".join(__table_referenced.split("_")[1:])
        count = 0
        while __table_referenced in list:
            count = count + 1
            __table_referenced = "{}{}".format(__table_referenced, count)
        list.append(__table_referenced)
        return __table_referenced

def Make(dir, db_user, db_password, db_host, db_port, db_database, db_ssl=False, db_ssl_ca=None, db_ssl_cert=None, db_ssl_key=None, date_format="%d/%m/%Y %H:%M:%S", field_types=None):

    _inflector = Inflector()

    print("\n [ Python Entity Model ] \n")

    root_path = dir
    model_path = "{}/model/".format(root_path)
    model_lib_path = "{}/model/lib/".format(root_path)

    # create model path
    if not os.path.exists(model_lib_path):
        os.mkdir(model_lib_path)
        print("\t\u2714 Creating Model Folder on Project")
    else:
        print("\t\u2714 Model Folder Exists")

    print("\t\u2714 Connecting Database...")

    # SQL EXECUTA SQL QUERY
    if not db_ssl:
        db = mariadb.connect(user=db_user, password=db_password,
                             host=db_host, port=db_port, database=db_database)
    else:
        db = mariadb.connect(user=db_user, password=db_password, host=db_host, port=db_port,
                             database=db_database, ssl_ca=db_ssl_ca, ssl_cert=db_ssl_cert, ssl_key=db_ssl_key)

    cursor = db.cursor(dictionary=True)

    print("\t\u2714 Fetching Tables")

    # REFLECTION TABELAS DO PROJETO
    cursor.execute("SELECT TABLE_NAME FROM information_schema.tables where table_schema='{database}' ORDER BY TABLE_NAME ASC".format(
        database=db_database))
    tables = cursor.fetchall()

    cursor.execute("SELECT DISTINCT information_schema.key_column_usage.constraint_name, information_schema.key_column_usage.table_name, information_schema.key_column_usage.column_name, information_schema.key_column_usage.referenced_table_name, information_schema.key_column_usage.referenced_column_name FROM information_schema.key_column_usage, information_schema.tables AS tables, information_schema.tables AS referenced_tables WHERE information_schema.key_column_usage.table_schema='{database}' AND tables.table_name = information_schema.key_column_usage.table_name AND referenced_tables.table_name = information_schema.key_column_usage.referenced_table_name AND information_schema.key_column_usage.referenced_table_name IS NOT NULL  ORDER BY TABLE_NAME ASC".format(
        database=db_database))
    relationships = cursor.fetchall()

    cursor.execute(
        "SELECT * FROM information_schema.columns where table_schema='{0}' ORDER BY TABLE_NAME ASC, ORDINAL_POSITION ASC".format(db_database))
    columns = cursor.fetchall()

    print("\t\u2714 Create Entities: \n")

    models = []

    for table in tables:

        referenced_table = []

        classname = _inflector.classify(table["TABLE_NAME"])

        table_relationships_one_to_one = [
            relationship for relationship in relationships if relationship["table_name"] == table["TABLE_NAME"]]

        table_relationships_many = [relationship for relationship in relationships if relationship["referenced_table_name"] == table["TABLE_NAME"]]

        table_relationships_one_to_n = []
        table_relationships_n_to_n = []

        # CHECK AND MAKE LIST FOR MANY TO MANY RELATION
        for many_relation in table_relationships_many:
            table_columns_of_many = [column for column in columns if column["TABLE_NAME"] == many_relation["table_name"]]
            # CHECK IF RELATION IS INTERMEDIATE OF MANY TO MANY
            # IF HAS 2 FIELDS AND ALL FIELDS ARE PRIMARY AND NOT NULL
            if len(table_columns_of_many) == 2 and all([col["COLUMN_KEY"] == 'PRI' and col["IS_NULLABLE"]=="NO" for col in table_columns_of_many]):

                points_of_rel = [relationship for relationship in relationships if relationship["table_name"] == many_relation['table_name']]
                for end_of_rel in points_of_rel:
                    if end_of_rel != many_relation:
                        table_relationships_n_to_n.append({
                                "name": end_of_rel["referenced_table_name"],
                                "reference": many_relation["referenced_column_name"],
                                "table": end_of_rel["referenced_table_name"],
                                "key": end_of_rel["referenced_column_name"],
                                "intermediate": many_relation["table_name"],
                                "ref_key": many_relation["column_name"],
                                "rel_key" : end_of_rel["column_name"],
                                "constraint_name": end_of_rel["constraint_name"]
                            })
            else:
                table_relationships_one_to_n.append(many_relation)

        table_columns = [
            column for column in columns if column["TABLE_NAME"] == table["TABLE_NAME"]]

        path_entitie = "{}/{}.py".format(model_lib_path, classname.lower())
        models.append("from model.lib.{} import {}".format(
            classname.lower(), classname))

        print("\t\t\u2714 Creating Model: {} : {}".format(
            table["TABLE_NAME"], classname))

        with open(path_entitie, "w") as entitie:
            entitie.write("# -*- coding: utf-8 -*-")
            entitie.write("\nfrom bravaorm.entity import *")

            entitie.write(f"\n\nclass {classname}(Entity):")
            entitie.write("\n\n\tdef __init__(cls, **kw):")

            pks = "{" + f"'pk' : {[pk['COLUMN_NAME'] for pk in table_columns if 'COLUMN_KEY' in pk and pk['COLUMN_KEY'] == 'PRI']}" + "}"
            entitie.write(f"\n\n\t\tcls.__metadata__ = {pks}")

            entitie.write("\n\n\t\t# FIELDS".format(pks))
            for table_column in table_columns:



                if field_types and table_column['COLUMN_NAME'] in field_types:
                    decorator = field_types[table_column['COLUMN_NAME']]
                    entitie.write(f"\n\t\tcls.{table_column['COLUMN_NAME']} = {decorator}")
                else:
                    decorator = "String"
                    if "tinyint" in table_column["COLUMN_TYPE"] or "bigint" in table_column["COLUMN_TYPE"] or "int" in table_column["COLUMN_TYPE"]:
                        decorator = "Int"
                    elif "decimal" in table_column["COLUMN_TYPE"]:
                        decorator = "Decimal"
                    elif "datetime" in table_column["COLUMN_TYPE"]:
                        decorator = "DateTime"
                    elif "float" in table_column["COLUMN_TYPE"]:
                        decorator = "Float"

                    if decorator == 'String' and table_column["CHARACTER_MAXIMUM_LENGTH"] != "NULL" and table_column["CHARACTER_MAXIMUM_LENGTH"] == 4294967295:
                        decorator = "Json"

                    colum_config = []
                    if "datetime" in table_column["COLUMN_TYPE"]:
                        colum_config.append("format='{}'".format(date_format))
                    if table_column["COLUMN_KEY"] == 'PRI':
                        colum_config.append("pk=True")
                    if table_column["COLUMN_KEY"] == "MUL":
                        colum_config.append("fk=True")
                    if table_column["EXTRA"] == 'auto_increment':
                        colum_config.append("auto_increment=True")
                    if table_column["IS_NULLABLE"] == "NO":
                        colum_config.append("not_null=True")
                    if table_column["CHARACTER_MAXIMUM_LENGTH"] != "NULL" and table_column["CHARACTER_MAXIMUM_LENGTH"] != None and decorator != 'Json':
                        colum_config.append(
                            "max=%s" % table_column["CHARACTER_MAXIMUM_LENGTH"])
                    if table_column["NUMERIC_PRECISION"] != None:
                        colum_config.append("precision=%s" %
                                            table_column["NUMERIC_PRECISION"])
                    if table_column["NUMERIC_SCALE"] != None:
                        colum_config.append("scale=%s" %
                                            table_column["NUMERIC_SCALE"])

                    entitie.write(f"\n\t\tcls.{table_column['COLUMN_NAME']} = {decorator}({', '.join(colum_config)})")


            if len(table_relationships_one_to_one) > 0:
                entitie.write("\n\n\t\t# One-to-One")

            for table_relationship in table_relationships_one_to_one:
                if table_relationship["referenced_table_name"] != table["TABLE_NAME"]:

                    __table_referenced = reference_name(table["TABLE_NAME"], referenced_table, table_relationship["referenced_table_name"], table_relationship["constraint_name"])


                    entitie.write(f"\n\t\tcls.{__table_referenced} = Obj(context=cls, keyname='{__table_referenced}', reference='{table_relationship['column_name']}', name='{_inflector.classify(table_relationship['referenced_table_name'])}', table='{table_relationship['referenced_table_name']}', key='{table_relationship['referenced_column_name']}')")


            if len(table_relationships_one_to_n) > 0:
                entitie.write("\n\n\t\t# One-to-many")

            for table_relationship in table_relationships_one_to_n:

                __table_referenced = reference_name(table["TABLE_NAME"], referenced_table, table_relationship["table_name"], table_relationship["constraint_name"])

                entitie.write(f"\n\t\tcls.{__table_referenced} = ObjList(context=cls, keyname='{__table_referenced}', reference='{table_relationship['referenced_column_name']}',  name='{_inflector.classify(table_relationship['table_name'])}', table='{table_relationship['table_name']}', key='{table_relationship['column_name']}')")

            if len(table_relationships_n_to_n) > 0:
                entitie.write("\n\n\t\t# Many-to-many")

            for table_relationship in table_relationships_n_to_n:

                __table_referenced = reference_name(table["TABLE_NAME"], referenced_table, table_relationship["name"], table_relationship["constraint_name"])

                entitie.write(f"\n\t\tcls.{__table_referenced} = ObjListOfMany(context=cls, keyname='{__table_referenced}', reference='{table_relationship['reference']}', name='{_inflector.classify(table_relationship['name'])}', intermediate='{table_relationship['intermediate']}', ref_key='{table_relationship['ref_key']}', rel_key='{table_relationship['rel_key']}', table='{table_relationship['table']}', key='{table_relationship['key']}')")

            entitie.write("\n\n\t\tsuper().__init__(**kw)")
            entitie.write("\n")

        # WRITE MODEL PY FILE
        path_model = "{}/{}.py".format(model_path, classname.lower())
        if not os.path.exists(path_model):
            with open(path_model, "w") as model_entitie:
                model_entitie.write("# -*- coding: utf-8 -*-")
                model_entitie.write(f"\nfrom bravaorm.entity.datatype import *")
                model_entitie.write(f"\nfrom .lib import {classname}")
                model_entitie.write(f"\n\nclass {classname}({classname}):")
                model_entitie.write("\n\n\tdef __init__(cls, **kw):")
                model_entitie.write(f"\n\t\treturn super({classname}, cls).__init__(**kw)")

    # init model path

    # INICIA PASTA LIB ENTITY __INIT__.PY
    with open("{}/__init__.py".format(model_lib_path), "w") as lib_init_file:
        lib_init_file.write("#-*- coding: utf-8 -*-\n\n")
        lib_init_file.write("\n".join(models))

    print("\n\t\u2714 LIB Init py folder")

    # INICIA PASTA MODEL __INIT__.PY
    with open("{}/__init__.py".format(model_path), "w") as lib_init_file:
        lib_init_file.write("#-*- coding: utf-8 -*-\n\n")
        lib_init_file.write("\n".join(models).replace('model.lib.', 'model.'))

    print("\n\t\u2714 Init py folder")

    print("\n\t\u2714 Done!\n\n")



# Copyright (c) 2006 Bermi Ferrer Martinez
#
# bermi a-t bermilabs - com
# See the end of this file for the free software, open source license (BSD-style).

from .languages.english import English
from .languages.spanish import Spanish
from .languages.portuguese import Portuguese


class Inflector(object):
    """
    Inflector for pluralizing and singularizing nouns.

    It provides methods for helping on creating programs
    based on naming conventions like on Ruby on Rails.
    """

    def __init__(self, language=Portuguese):
        assert callable(language), "language should be a callable obj"
        self._language = language()

    def pluralize(self, word):
        '''Pluralizes nouns.'''
        return self._language.pluralize(word)

    def singularize(self, word):
        '''Singularizes nouns.'''
        return self._language.singularize(word)

    def conditional_plural(self, number_of_records, word):
        '''Returns the plural form of a word if first parameter is
        greater than 1.
        '''
        return self._language.conditional_plural(number_of_records, word)

    def titleize(self, word, uppercase=''):
        '''Converts an underscored or CamelCase word into a sentence.
        The titleize function converts text like "WelcomePage",
        "welcome_page" or  "welcome page" to this "Welcome Page".
        If the "uppercase" parameter is set to 'first' it will only
        capitalize the first character of the title.
        '''
        return self._language.titleize(word, uppercase)

    def camelize(self, word):
        '''Returns given word as CamelCased.
        Converts a word like "send_email" to "SendEmail". It
        will remove non alphanumeric characters from the word, so
        "who's online" will be converted to "WhoSOnline"
        '''
        return self._language.camelize(word)

    def underscore(self, word):
        ''' Converts a word "into_it_s_underscored_version"
        Convert any "CamelCased" or "ordinary Word" into an
        "underscored_word".

        This can be really useful for creating friendly URLs.
        '''
        return self._language.underscore(word)

    def humanize(self, word, uppercase=''):
        '''Returns a human-readable string from word
        Returns a human-readable string from word, by replacing
        underscores with a space, and by upper-casing the initial
        character by default.

        If you need to uppercase all the words you just have to
        pass 'all' as a second parameter.
        '''
        return self._language.humanize(word, uppercase)

    def variablize(self, word):
        '''Same as camelize but first char is lowercased
        Converts a word like "send_email" to "sendEmail". It
        will remove non alphanumeric character from the word, so
        "who's online" will be converted to "whoSOnline"'''
        return self._language.variablize(word)

    def tableize(self, class_name):
        ''' Converts a class name to its table name according to rails
        naming conventions. Example. Converts "Person" to "people".
        '''
        return self._language.tableize(class_name)

    def classify(self, table_name):
        '''Converts a table name to its class name according to rails
        naming conventions. Example: Converts "people" to "Person"
        '''
        return self._language.classify(table_name)

    def ordinalize(self, number):
        '''Converts number to its ordinal form.
        This method converts 13 to 13th, 2 to 2nd ...
        '''
        return self._language.ordinalize(number)

    def unaccent(self, text):
        '''Transforms a string to its unaccented version.
        This might be useful for generating "friendly" URLs'''
        return self._language.unaccent(text)

    def urlize(self, text):
        '''Transform a string its unaccented and underscored
        version ready to be inserted in friendly URLs.
        '''
        return self._language.urlize(text)

    def demodulize(self, module_name):
        return self._language.demodulize(module_name)

    def modulize(self, module_description):
        return self._language.modulize(module_description)

    def foreign_key(self, class_name,
                    separate_class_name_and_id_with_underscore=1):
        ''' Returns class_name in underscored form, with "_id" tacked on at the end.
        This is for use in dealing with the database.
        '''
        return self._language.foreign_key(
            class_name,
            separate_class_name_and_id_with_underscore
        )

    def conditionalPlural(self, number_of_records, word):
        '''Deprecated, alias of #conditional_plural for backwards
        compatibility.
        '''
        return self.conditional_plural(number_of_records, word)

    def foreignKey(self, class_name, separate_class_name_and_id_with_underscore=1):
        '''Deprecated, alias of #foreign_key for backwards compatibility.'''
        return self.foreign_key(class_name,
                                separate_class_name_and_id_with_underscore)

# Copyright (c) 2006 Bermi Ferrer Martinez
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software to deal in this software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of this software, and to permit
# persons to whom this software is furnished to do so, subject to the following
# condition:
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THIS SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THIS SOFTWARE.



# -*- coding: utf-8 -*-

from bravaorm.entity.datatype import *

__methods__ = ['toJSON', 'load', 'add']

class Entity():

    def __init__(self, aliases=None, context=None, **kw):
        self.__metadata__['data'] = {}
        self.__metadata__['aliases'] = {}
        self.__metadata__['relasionships'] = {}
        self.__metadata__['status'] = 'created'
        self.__context__ = context
        self.load(**kw)
        if aliases:
            for c, k in aliases:
                if not k in self.__metadata__['aliases']:
                    self.__metadata__['aliases'][k] = kw[k]

    def load(self, **kw):
        if len(kw) > 0:
            for k in self.__dict__:
                if not k.startswith("__"):
                    if k in kw:
                        if self[k].__class__.__name__.startswith("Obj"):
                            self.add(k, kw[k])
                        else:
                            self[k].value = kw[k]
                            self.__metadata__['data'][k] = self[k].value
                    elif self[k].__class__.__name__.startswith("Obj"):
                        group_data = {key.replace(f"{k}.",''):value for key, value in kw.items() if key.startswith(f"{k}.")}
                        if len(group_data) > 0:
                            self.add(k, group_data)                   

    def add(self, key=None, data=None):
        if key and data and (isinstance(data, dict) or isinstance(data, list)):
            if "List" in self[key].__class__.__name__:
                if not key in self.__metadata__['relasionships']:
                    self.__metadata__['relasionships'][key]=[]
                self.__metadata__['relasionships'][key].extend(data if isinstance(data, list) else [data])
                self[key].value.extend([self[key].type(context={'entity':self ,'key':key}, **item) for item in data if any(item.values())] if isinstance(data, list) else [self[key].type(context={'entity':self ,'key':key}, **data)] if any(data.values()) else [])
            else:
                data = data[0] if isinstance(data, list) else data
                if any(data.values()):
                    self.__metadata__['relasionships'][key] = data
                    self[key].value = self[key].type(context={'entity':self ,'key':key} , **data)
        else:
            raise Exception("entity.add requires key and dict of object data")

    def __getitem__(self, field):
        return super().__getattribute__(field) if hasattr(self, field) else self.__metadata__['aliases'][field]

    def __getattribute__(self, field):
        if field.startswith("__") or field in __methods__:
            return super().__getattribute__(field)
        else:
            return super().__getattribute__(field).value

    def __setattr__(self, item, value):
        try:
            if not item.startswith("__") and not "entity.datatype" in str(value.__class__):
                if self[item]:
                    if hasattr(value, '__context__') and not value.__context__:
                        value.__context__ = self
                    self[item].value = value
                    self.__metadata__['data'][item] = self[item].value.toJSON() if hasattr(self[item].value, 'toJSON') else self[item].value
                    self.__metadata__['status'] = 'modified'
                    if self.__context__:
                        _context = self.__context__['entity']
                        _context_key = self.__context__['key']
                        if isinstance(_context[_context_key].value, list):
                            index = _context[_context_key].value.index(self)
                            _context.__metadata__['relasionships'][_context_key][index] = self.__metadata__['data']
                        else:
                            _context.__metadata__['relasionships'][_context_key] = self.__metadata__['data']
                else:
                    super().__setattr__(item, value)
            else:
                super().__setattr__(item, value)
        except Exception as e:
            raise Exception(f'{item} {e}')

    def toJSON(self):
        try:
            return {**self.__metadata__['data'], **self.__metadata__['relasionships'], **self.__metadata__['aliases']}
        except Exception as e:
            raise e



#!/usr/bin/env python
# -*- coding: latin-1 -*-
# Copyright (c) 2006 Bermi Ferrer Martinez
# Copyright (c) 2006 Carles Sadurn� Anguita
#
# bermi a - t bermilabs - com
#
# See the end of this file for the free software, open source license (BSD - style).

import re
from .base import Base

class Spanish(Base):
    '''
    Inflector for pluralize and singularize Spanish nouns.
    '''

    def pluralize(self, word):
        '''Pluralizes Spanish nouns.'''
        rules = [
            [r'(?i)([aeiou])x$', r'\1x'],  # This could fail if the word is oxytone.
            [r'(?i)([�����])([ns])$', r'|1\2es'],
            [r'(?i)(^[bcdfghjklmn�pqrstvwxyz]*)an$', r'\1anes'],  # clan - >clanes
            [r'(?i)([�����])s$', r'|1ses'],
            [r'(?i)(^[bcdfghjklmn�pqrstvwxyz]*)([aeiou])([ns])$', r'\1\2\3es'],  # tren - >trenes
            [r'(?i)([aeiou���])$', r'\1s'],  # casa - >casas, padre - >padres, pap� - >pap�s
            [r'(?i)([aeiou])s$', r'\1s'],  # atlas - >atlas, virus - >virus, etc.
            [r'(?i)([��])(s)$', r'|1\2es'],  # ingl�s - >ingleses
            [r'(?i)z$', 'ces'],  # luz - >luces
            [r'(?i)([��])$', r'\1es'],  # ceut� - >ceut�es, tab� - >tab�es
            [r'(?i)(ng|[wckgtp])$', r'\1s'],  # Anglicismos como puenting, frac, crack, show (En que casos podr�a fallar esto?)
            [r'(?i)$', 'es']	 # ELSE +es (v.g. �rbol - >�rboles)
        ]

        uncountable_words = ['tijeras', 'gafas', 'vacaciones', 'v�veres', 'd�ficit']
        ''' In fact these words have no singular form: you cannot say neither
        "una gafa" nor "un v�vere". So we should change the variable name to
        onlyplural or something alike.'''

        irregular_words = {
            'pa�s': 'pa�ses',
            'champ�': 'champ�s',
            'jersey': 'jers�is',
            'car�cter': 'caracteres',
            'esp�cimen': 'espec�menes',
            'men�': 'men�s',
            'r�gimen': 'reg�menes',
            'curriculum': 'curr�culos',
            'ultim�tum': 'ultimatos',
            'memor�ndum': 'memorandos',
            'refer�ndum': 'referendos'
        }

        lower_cased_word = word.lower()

        for uncountable_word in uncountable_words:
            if lower_cased_word[- 1 * len(uncountable_word):] == uncountable_word:
                return word

        for irregular in irregular_words.keys():
            match = re.search('(?i)(' + irregular + ')$', word, re.IGNORECASE)
            if match:
                return re.sub('(?i)' + irregular + '$', match.expand('\\1')[0] + irregular_words[irregular][1:], word)

        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)

            if match:
                groups = match.groups()
                replacement = rules[rule][1]
                if re.match('\|', replacement):
                    for k in range(1, len(groups)):
                        replacement = replacement.replace('|' + str(k), self.string_replace(groups[k - 1], '����������', 'AEIOUaeiou'))

                result = re.sub(rules[rule][0], replacement, word)
                # Esto acentua los sustantivos que al pluralizarse se convierten en esdr�julos como esm�quines, j�venes...
                match = re.search('(?i)([aeiou]).{1, 3}([aeiou])nes$', result)

                if match and len(match.groups()) > 1 and not re.search('(?i)[�����]', word):
                    result = result.replace(match.group(0), self.string_replace(match.group(1), 'AEIOUaeiou', '����������') + match.group(0)[1:])

                return result

        return word

    def singularize(self, word):
        '''Singularizes Spanish nouns.'''

        rules = [
            ['(?i)^([bcdfghjklmn�pqrstvwxyz]*)([aeiou])([ns])es$', '\\1\\2\\3'],
            ['(?i)([aeiou])([ns])es$',  '~1\\2'],
            ['(?i)oides$',  'oide'],  # androides - >androide
            ['(?i)(ces)$/i', 'z'],
            ['(?i)(sis|tis|xis) + $',  '\\1'],  # crisis, apendicitis, praxis
            ['(?i)(�)s$',  '\\1'],  # beb�s - >beb�
            ['(?i)([^e])s$',  '\\1'],  # casas - >casa
            ['(?i)([bcdfghjklmn�prstvwxyz]{2, }e)s$', '\\1'],  # cofres - >cofre
            ['(?i)([gh�pv]e)s$', '\\1'],  # 24 - 01 llaves - >llave
            ['(?i)es$', '']  # ELSE remove _es_  monitores - >monitor
        ]

        uncountable_words = ['paraguas', 'tijeras', 'gafas', 'vacaciones', 'v�veres', 'lunes', 'martes', 'mi�rcoles', 'jueves', 'viernes', 'cumplea�os', 'virus', 'atlas', 'sms']

        irregular_words = {
            'jersey': 'jers�is',
            'esp�cimen': 'espec�menes',
            'car�cter': 'caracteres',
            'r�gimen': 'reg�menes',
            'men�': 'men�s',
            'r�gimen': 'reg�menes',
            'curriculum': 'curr�culos',
            'ultim�tum': 'ultimatos',
            'memor�ndum': 'memorandos',
            'refer�ndum': 'referendos',
            's�ndwich': 's�ndwiches'
        }

        lower_cased_word = word.lower()

        for uncountable_word in uncountable_words:
            if lower_cased_word[-1 * len(uncountable_word):] == uncountable_word:
                return word

        for irregular in irregular_words.keys():
            match = re.search('(' + irregular + ')$', word, re.IGNORECASE)
            if match:
                return re.sub('(?i)' + irregular + '$', match.expand('\\1')[0] + irregular_words[irregular][1:], word)

        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)
            if match:
                groups = match.groups()
                replacement = rules[rule][1]
                if re.match('~', replacement):
                    for k in range(1, len(groups)):
                        replacement = replacement.replace('~' + str(k), self.string_replace(groups[k - 1], 'AEIOUaeiou', '����������'))

                result = re.sub(rules[rule][0], replacement, word)
                # Esta es una posible soluci�n para el problema de dobles acentos. Un poco guarrillo pero funciona
                match = re.search('(?i)([�����]).*([�����])', result)

                if match and len(match.groups()) > 1 and not re.search('(?i)[�����]', word):
                    result = self.string_replace(result, '����������', 'AEIOUaeiou')

                return result

        return word


# Copyright (c) 2006 Bermi Ferrer Martinez
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software to deal in this software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of this software, and to permit
# persons to whom this software is furnished to do so, subject to the following
# condition:
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THIS SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THIS SOFTWARE.



#!/usr/bin/env python

# Copyright (c) 2006 Bermi Ferrer Martinez
# bermi a-t bermilabs - com
#
# See the end of this file for the free software, open source license (BSD-style).

import re
from .base import Base

class Portuguese(Base):
    """
    Inflector for pluralize and singularize English nouns.

    This is the default Inflector for the Inflector obj
    """

    def pluralize(self, word) :
        '''Pluralizes English nouns.'''

        rules = [
            ['(?i)r$', 'res'],
            ['(?i)m$', 'ns'],
            ['(?i)il$', 'is'],
            ['(?i)l$', 'is'],
            ['(?i)(ao)$', 'oes'],
            ['(?i)([aeiou])$', '\\1s']
        ]


        uncountable_words = [
            'lapis',
            'onibus',
            'virus',
            'caridade',
            'bondade',
            'fe',
            'ouro',
            'prata',
            'bronze',
            'brisa',
            'oxigenio',
            'fome',
            'sede',
            'po',
            'plebe',
            'neve',
            'lenha',
            'cristianismo',
            'nazismo',
            'sinceridade',
            'lealdade',
            'status'
        ]

        irregular_words = {
            'bookmark': 'bookmarks',
            'direct': 'directs',
            'inbox': 'inboxes',
            'session': 'sessions',
            'fan': 'fans',
            'feed': 'feeds',
            'app':'apps',
            'server': 'servers',
            'mediaserver': 'mediaservers',
            'streamer': 'streamers',
            'mal': 'males',
            'consul': 'consules',
            'mel': 'meis',
            'cal': 'cais',
            'aval': 'avais',
            'mol': 'mois',
            'til': 'tis',
            'projetil': 'projeteis',
            'facil': 'faceis',
            'dificil': 'dificeis',
            'fossil': 'fosseis',
            'cep': 'ceps',
            'log': 'logs',
            'banner': 'banners',
            'faq': 'faqs',
            'newsletter': 'newsletters',
            'vip': 'vips',
            'deploy': 'deploys',
            'mes': 'meses'
        }

        lower_cased_word = word.lower();

        for uncountable_word in uncountable_words:
            if lower_cased_word[-1*len(uncountable_word):] == uncountable_word :
                return word

        for irregular in irregular_words.keys():
            match = re.search('('+irregular+')$',word, re.IGNORECASE)
            if match:
                return re.sub('(?i)'+irregular+'$', match.expand('\\1')[0]+irregular_words[irregular][1:], word)

        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)
            if match :
                groups = match.groups()
                for k in range(0,len(groups)) :
                    if groups[k] == None :
                        rules[rule][1] = rules[rule][1].replace('\\'+str(k+1), '')

                return re.sub(rules[rule][0], rules[rule][1], word)

        return word


    def singularize_word(self, word) :
        '''Singularizes English nouns.'''

        rules = [
            ['((?i)ns)$', 'm'],
            ['((?i)[e][i]s)$', 'el'],
            ['(?i)(ais)$', 'al'],
            ['(?i)([i]s)$', 'il'],
            ['(?i)(oe)s$', 'ao'],
            ['(?i)(c)oes$', '\\1ao'],
            ['(?i)(r)es$', '\\1'],
            ['(?i)(z)es$', '\\1'],
            ['(?i)(s)es$', '\\1'],
            ['(?i)(le)ns$', '\\1n'],
            ['(?i)(de)ns$', '\\1n'],
            ['(?i)([aeou])is$', 'il'],
            ['(?i)([aeiou])s$', '\\1'],
            ['(?i)ns$', 'm'],
        ];

        uncountable_words = [
            'lapis',
            'onibus',
            'virus',
            'caridade',
            'bondade',
            'fe',
            'ouro',
            'prata',
            'bronze',
            'brisa',
            'oxigenio',
            'fome',
            'sede',
            'po',
            'plebe',
            'neve',
            'lenha',
            'cristianismo',
            'nazismo',
            'sinceridade',
            'lealdade',
            'status'
        ]

        irregular_words = {
            'bookmarks': 'bookmark',
            'directs': 'direct',
            'inboxes': 'inbox',
            'sessions': 'session',
            'fans': 'fan',
            'feeds': 'feed',
            'apps':'app',
            'servers': 'server',
            'mediaservers': 'mediaserver',
            'streamers': 'streamer',
            'males': 'mal',
            'consules': 'consul',
            'meis': 'mel',
            'cais': 'cal',
            'avais': 'aval',
            'mois': 'mol',
            'tis': 'til',
            'projeteis': 'projetil',
            'faceis': 'facil',
            'dificeis': 'dificil',
            'fosseis': 'fossil',
            'ceps': 'cep',
            'logs': 'log',
            'banners': 'banner',
            'faqs': 'faq',
            'newsletters': 'newsletter',
            'vips': 'vip',
            'deploys': 'deploy',
            'meses': 'mes'
        }


        lower_cased_word = word.lower();

        for uncountable_word in uncountable_words:
            if lower_cased_word[-1*len(uncountable_word):] == uncountable_word :
                return word

        for irregular in irregular_words.keys():
            match = re.search('('+irregular+')$',word, re.IGNORECASE)
            if match:
                return re.sub('(?i)'+irregular+'$', match.expand('\\1')[0]+irregular_words[irregular][1:], word)


        for rule in range(len(rules)):
            match = re.search(rules[rule][0], word, re.IGNORECASE)
            if match :
                groups = match.groups()
                for k in range(0,len(groups)) :
                    if groups[k] == None :
                        rules[rule][1] = rules[rule][1].replace('\\'+str(k+1), '')

                return re.sub(rules[rule][0], rules[rule][1], word)

        return word

    def singularize(self, word):
        '''Singularizes each word'''
        words = word.split('_')
        words_done = []
        for word in words:
            words_done.append(self.singularize_word(word))
        return '_'.join(words_done)



# Copyright (c) 2006 Bermi Ferrer Martinez
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software to deal in this software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of this software, and to permit
# persons to whom this software is furnished to do so, subject to the following
# condition:
#
# THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THIS SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THIS SOFTWARE.
